/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "HW_Configs.h"
#include "stm32f0xx_it.h"
#include "tic32.h"

//   ----------------------------------

//   -----------------------------------------------------------
int16_t AdcData[LENSMPL];
uint32_t Flag;
uint8_t State = 1; // 0 - , 1 - 20R, 2 - 10R, 3 - 6.7R, 4 - .
uint32_t Data, Ref;

uint8_t Day = 0, Hour = 0, Min = 0, Sec = 0;
uint8_t DayS = 0, HourS = 0, MinS = 0, SecS = 0;
uint8_t SecU, MinU;
int32_t Uu = 0, Ii = 0;
double AmA = 0, Aw = 0;
char str[32] = "";
float Ff;

//    ----------------------------------------


////////////////////////////////   /////////////////////////////////////////
int main(void)
{
//-------------------------------------------------------------------------------------
//	 stm32f0xx.h  -   ,   ,  
//	 system_stm32f0xx.c :
//		SystemInit() -  HSI,  .
//		SetSysClock() -   HSE.   !
//-------------------------------------------------------------------------------------
	__disable_irq();
	SYS_Config(); // ,   
	SCK_Config(); // 
	GPIO_Config(); //  
	I2C_Config(); //   I2C1
	ADC_Config(); //  
	DMA_Config();
	NVIC_Config(); //    

	GPIOA->ODR &= ~GPIO_Pin_9; // "0"    
	CH_OFF(); //   

	DELAY_US(50000);
	__enable_irq();
	TIM_Config(); // 

	LCD_Init(100); //  
	LCD_Clear(0);
	LCD_Cursor(18*6, 3); LCD_String("r3"); //  
	L20_ON(); //  

	while (1) //  
	{
		if (Flag & BIT(READY)) //  100  -----------------------------------------------
		{
			Flag &= ~BIT(READY); //  

			Uu = ((uint64_t)Data * CAL_MCU_MV * VREFINT_CAL * 2)/ 4095 / Ref; // mV  
			Ii = 0; // 

			if ((State > 0)&&(State < 4)) // 
			{
				if (Uu < TH_END_MV) //  
				{
					State = 4; //  
					L10_OFF();
					L20_OFF();
					CH_ON(); //  
				}
				else
				{
					CH_OFF(); //  

					if (State & BIT(0))
					{
						L20_ON();
						Ii += ((float)Uu / R20_LOAD); // m  
					}
					else L20_OFF();

					if (State & BIT(1))
					{
						L10_ON();
						Ii += ((float)Uu / R10_LOAD); // m  
					}
					else L10_OFF();

					LED_ON(); //  
					LCD_Cursor(16*6, 2); LCD_String("-DIS");
				}
			}
			else if (State == 4) // 
			{
				L10_OFF();
				L20_OFF();
				if (Uu > CA_END_MV) //  
				{
					CH_OFF(); //  
					State = 0; // 
					LCD_Cursor(16*6, 2); LCD_String("    "); // off
					LED_ON(); // 
				}
				else
				{
					CH_ON(); //  
					LCD_Cursor(16*6, 2); LCD_String("+CHR");
					if (Sec != SecU) LED_ON(); //    
					SecU = Sec;
				}
			}

			AmA += Ii;
			Aw += 1E-3 *(Uu * Ii); //    100   

			if (Min != MinU) //   
			{
				LCD_Init(100); //  
				LCD_Clear(0);
			}
			MinU = Min;

			Flag |= BIT(DISP); // 
		}


		if (Flag & BIT(DISP)) //   ----------------------------------------------
		{
			Flag &= ~BIT(DISP); //  

			LCD_Cursor(1*6, 0); sprintf(str, "%4umV -%02u/%02u:%02u:%02u", Uu, DayS, HourS, MinS, SecS); LCD_String(str);

			if (State == 4)
			{
				LCD_Cursor(1*6, 1); LCD_String("+330");
			}
			else
			{
				LCD_Cursor(1*6, 1); sprintf(str, "%3im  ", -Ii); LCD_String(str);
			}

			LCD_Cursor(8*6, 1); sprintf(str, "+%02u/%02u:%02u:%02u", Day, Hour, Min, Sec); LCD_String(str);
			LCD_Cursor(1*6, 2); sprintf(str, "%10.2fm* ", AmA/(60*60*10)); LCD_String(str);
			LCD_Cursor(1*6, 3); sprintf(str, "%10.2f* ", Aw/(60*60*10)); LCD_String(str);

			if (State != 0) LED_OFF(); //     
		}


		if (Flag & BIT(KEY1_P)) //   ---------------------------------------------------
		{
			Flag &= ~BIT(KEY1_P); //  

			if ((State > 0)&&(State < 4)) // 
			{
				State = 4;
				LCD_Cursor(16*6, 2); LCD_String("+CHR");
			}
			else
			{
				State = 1;
				LCD_Cursor(16*6, 2); LCD_String("-DIS");
			}
		}


		if (Flag & BIT(KEY1_DP)) //    ---------------------------------------------------
		{
			Flag &= ~BIT(KEY1_DP); //  

			if ((State > 0)&&(State < 4)) // 
			{
				State++; //   
				if (State >= 4) State = 1;
			}
		}


		if (Flag & BIT(KEY1_U)) //   ---------------------------------------------------
		{
			Flag &= ~BIT(KEY1_U); //  

			if ((State > 0)&&(State < 4)) // 
			{
				SecS = 0;
				MinS = 0;
				HourS = 0;
				DayS = 0;
			}
			else if (State == 4) // 
			{
				Sec = 0;
				Min = 0;
				Hour = 0;
				Day = 0;
			}
		}


		if (Flag & BIT(SEC)) //  --------------------------------------------------
		{
			Flag &= ~BIT(SEC); //  

			if ((State > 0)&&(State < 4)) // 
			{
				if (++SecS >= 60)
				{
					SecS = 0;
					if (++MinS >= 60)
					{
						MinS = 0;
						if (++HourS >= 24)
						{
							HourS = 0;
							DayS++;
						}
					}
				}
			}
			else if (State == 4) // 
			{
				if (++Sec >= 60)
				{
					Sec = 0;
					if (++Min >= 60)
					{
						Min = 0;
						if (++Hour >= 24)
						{
							Hour = 0;
							Day++;
						}
					}
				}
			}
		}

#ifdef W_DOG			//  
		IWDG_ReloadCounter(); //  
#endif
	}
}


/////////////    //////////////////////////////////////////////
void Delay_ms(uint32_t ms)
{
	ms *= SystemCoreClock / 6000; //  Optimize Level2 + for Time
	while (ms-- != 0) {__asm(" nop");}
}


/////////////////////////////////////////////////////////////////////////////////
#ifdef  USE_FULL_ASSERT
void assert_failed(uint8_t* file, uint32_t line){
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  while (1)  {  }
}
#endif
