// 

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f0xx_it.h"

//   ------------------------------------------------------
extern uint32_t Flag;
extern uint32_t Data, Ref;
extern int16_t AdcData[];
extern uint8_t Day, Hour, Min, Sec;

//   -----------------------------------------------------------


//    ----------------------------------------
void NMI_Handler(void);
void HardFault_Handler(void);
void SVC_Handler(void);
void PendSV_Handler(void);
void SysTick_Handler(void);
void DMA1_Channel1_IRQHandler(void);

/******************************************************************************/
/*            Cortex-M0 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief  This function handles NMI exception.
  */
void NMI_Handler(void)
{
#ifdef EXT_QUARZ //  
	Delay_ms(120);
	NVIC_SystemReset();
#endif
}

/**
  * @brief  This function handles Hard Fault exception.
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles SVCall exception.
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles PendSVC exception.
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler.
  */
void SysTick_Handler(void)
{
	static uint16_t Counter_KX = 0, Counter_KDouble = 0;

	if (!K1_GET()) // 
	{
		if (Counter_KX < 10000) Counter_KX++; //     
		if (Counter_KX == KUPRESS)  // 
		{
			Flag |= BIT(KEY1_U);
			Counter_KDouble = 0; //      
		}
	}
	else  // 
	{
		if ((Counter_KX > KDEBOUNCE)&&(Counter_KX < KUPRESS)) //    (     ) 
		{
			if (Counter_KDouble == 0) //   
			{
				Counter_KDouble = 1; //      
			}
			else if ((Counter_KDouble > KDEBOUNCE)&&(Counter_KDouble < KDOUBLEMAX)) //            
			{
				Flag |= BIT(KEY1_DP);
				Counter_KDouble = 0; //      
			}
		}
		Counter_KX = 0;
	}
	if (Counter_KDouble != 0) //      
	{
		Counter_KDouble++;
		if (Counter_KDouble >= KDOUBLEMAX) //      
		{
			Flag |= BIT(KEY1_P); // 
			Counter_KDouble = 0; //      
			Counter_KX = 0; //   ,  
		}
	}
}


//=========================================================================================================
void DMA1_Channel1_IRQHandler(void)
{
	static uint8_t	Timer = 0;
	static uint32_t	Adata = 0;
	static uint32_t	Aref = 0;
	int16_t i, j;

//	LEDR_ON();

	//       -   ,  
	if (DMA1->ISR & DMA_ISR_HTIF1)  //   
	{
		i = 0;
		j = LENSMPL/2;
	}
	else  //   
	{
		i = LENSMPL/2;
		j = LENSMPL;
	}

	DMA1->IFCR = DMA1_IT_GL1; //   

	while(i < j) //       
	{
		// -------------------------------------------------------------
		Adata += AdcData[i++];
		Aref += AdcData[i++];
	}

	if (j == LENSMPL) //    
	{
		Data = Adata;
		Ref = Aref;

		Adata = 0;
		Aref = 0;

		Flag |= BIT(READY); // ,   
	}
	else
	{
		if (++Timer >= 10) // 
		{
			Timer = 0;
			Flag |= BIT(SEC);
		}
	}

//	LEDR_OFF();
}







