#include "main.h"
#include "tic32.h"
#include "fonts.h"

//   ------------------------------------------------------

//   -----------------------------------------------------------

//    ----------------------------------------

/////////////////////////////  LCD  ///////////////////////////////////////
void LCD_Init(uint8_t Cont)
{
	PIN_LCDRES_OFF();
	Delay_ms(1);
	PIN_LCDRES_ON();
	Delay_ms(1);

	while(I2C_GetFlagStatus(I2C1, I2C_ISR_BUSY) != RESET){}; //   
	//  : , ,  24 , , 
	I2C_TransferHandling(I2C1, ADDR_LCD, 24, I2C_AutoEnd_Mode, I2C_Generate_Start_Write);

	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,0x01); //    
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,BIN(00100000)); // Enable chip,  
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,BIN(00001001)); // page select 
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,BIN(00001100)); // Display Mode: NORMAL 
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,BIN(00000101)); // MUX-rate: 1/34 
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,BIN(00010100)); // Bias system: 3 
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,0x01); //    
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,BIN(00001010)); // page select 
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,BIN(00001010)); // voltage multiplication factor: 2*V 
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,BIN(00100000)); // temperature coefficient: 0 
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();
	if (Cont & BIT(7)) I2C_SendData(I2C1,BIN(00000111)); // Vlcd - HIGH; Voltage multiplier ON 
	else I2C_SendData(I2C1,BIN(00000101)); // Vlcd - LOW; Voltage multiplier ON 
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,(BIT(7)|Cont)); // Vlcd set 
	WAIT_STOP();
}


//----------------------------------------------------------------------------	
void LCD_Clear(uint8_t typ)//  , tip==0 , tip==1 
{
	uint16_t b;

	if (typ != 0) typ = 0xFF;

	while(I2C_GetFlagStatus(I2C1, I2C_ISR_BUSY) != RESET){}; //   

	//  : , ,  1 , ,  
	I2C_TransferHandling(I2C1, ADDR_LCD, 1, I2C_Reload_Mode, I2C_Generate_Start_Write);
	WAIT_TXIS(); I2C_SendData(I2C1,DISP_LCD); //  
	while(I2C_GetFlagStatus(I2C1, I2C_ISR_TCR) == RESET){}; //   TCR    

	// 640   (255+255+130)

	//  :  255 , ,  
	I2C_TransferHandling(I2C1, ADDR_LCD, 255, I2C_Reload_Mode, I2C_No_StartStop);
	for (b=0; b<255; b++)
	{
		WAIT_TXIS(); I2C_SendData(I2C1, typ); //    
	}
	while(I2C_GetFlagStatus(I2C1, I2C_ISR_TCR) == RESET){}; //   TCR    

	//  :  255 , ,  
	I2C_TransferHandling(I2C1, ADDR_LCD, 255, I2C_Reload_Mode, I2C_No_StartStop);
	for (b=0; b<255; b++)
	{
		WAIT_TXIS(); I2C_SendData(I2C1, typ); //    
	}
	while(I2C_GetFlagStatus(I2C1, I2C_ISR_TCR) == RESET){}; //   TCR    

	//  :  130 , ,  
	I2C_TransferHandling(I2C1, ADDR_LCD, 130, I2C_AutoEnd_Mode, I2C_No_StartStop);
	for (b=0; b<130; b++)
	{
		WAIT_TXIS(); I2C_SendData(I2C1, typ);
	}
	WAIT_STOP();
}


//----------------------------------------------------------------------------	
void LCD_Cursor(uint8_t X,uint8_t Y)//   
{

	while(I2C_GetFlagStatus(I2C1, I2C_ISR_BUSY) != RESET){}; //   
	//  : , ,  6 , , 
	I2C_TransferHandling(I2C1, ADDR_LCD, 6, I2C_AutoEnd_Mode, I2C_Generate_Start_Write);

	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,0x01); //    
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,BIN(01000000)+Y); // set Y-
	WAIT_TXIS();I2C_SendData(I2C1,COM_LCD);WAIT_TXIS();I2C_SendData(I2C1,BIN(10000000)+X); // set X-
	WAIT_STOP();
}


//----------------------------------------------------------------------------	
void LCD_SymbolBig(uint8_t codv, uint8_t X) //   16x32     X
{
	uint8_t ftmp[16]; //    (      )
	uint8_t a;
	uint16_t dv;

	dv = codv * 64;
	LCD_Cursor(X, 0); //   
	a = 0;
	while(a < 8)
	{
		ftmp[a] = 0;
		if (font16x32[dv+ 0] & BIT(7-a)) ftmp[a] |= BIT(0);
		if (font16x32[dv+ 2] & BIT(7-a)) ftmp[a] |= BIT(1);
		if (font16x32[dv+ 4] & BIT(7-a)) ftmp[a] |= BIT(2);
		if (font16x32[dv+ 6] & BIT(7-a)) ftmp[a] |= BIT(3);
		if (font16x32[dv+ 8] & BIT(7-a)) ftmp[a] |= BIT(4);
		if (font16x32[dv+10] & BIT(7-a)) ftmp[a] |= BIT(5);
		if (font16x32[dv+12] & BIT(7-a)) ftmp[a] |= BIT(6);
		if (font16x32[dv+14] & BIT(7-a)) ftmp[a] |= BIT(7);
		ftmp[a+8] = 0;
		if (font16x32[dv+ 1] & BIT(7-a)) ftmp[a+8] |= BIT(0);
		if (font16x32[dv+ 3] & BIT(7-a)) ftmp[a+8] |= BIT(1);
		if (font16x32[dv+ 5] & BIT(7-a)) ftmp[a+8] |= BIT(2);
		if (font16x32[dv+ 7] & BIT(7-a)) ftmp[a+8] |= BIT(3);
		if (font16x32[dv+ 9] & BIT(7-a)) ftmp[a+8] |= BIT(4);
		if (font16x32[dv+11] & BIT(7-a)) ftmp[a+8] |= BIT(5);
		if (font16x32[dv+13] & BIT(7-a)) ftmp[a+8] |= BIT(6);
		if (font16x32[dv+15] & BIT(7-a)) ftmp[a+8] |= BIT(7);
		a++;
	}
	while(I2C_GetFlagStatus(I2C1, I2C_ISR_BUSY) != RESET){}; //   
	//  : , ,  1+16 , , 
	I2C_TransferHandling(I2C1, ADDR_LCD, 17, I2C_AutoEnd_Mode, I2C_Generate_Start_Write);
	WAIT_TXIS(); I2C_SendData(I2C1, DISP_LCD); //  
	for(a = 0; a < 16; a++) 
	{
		WAIT_TXIS(); I2C_SendData(I2C1, ftmp[a]);
	}
	WAIT_STOP();

	dv += 16; //  
	LCD_Cursor(X, 1); //   
	a = 0;
	while(a < 8)
	{
		ftmp[a] = 0;
		if (font16x32[dv+ 0] & BIT(7-a)) ftmp[a] |= BIT(0);
		if (font16x32[dv+ 2] & BIT(7-a)) ftmp[a] |= BIT(1);
		if (font16x32[dv+ 4] & BIT(7-a)) ftmp[a] |= BIT(2);
		if (font16x32[dv+ 6] & BIT(7-a)) ftmp[a] |= BIT(3);
		if (font16x32[dv+ 8] & BIT(7-a)) ftmp[a] |= BIT(4);
		if (font16x32[dv+10] & BIT(7-a)) ftmp[a] |= BIT(5);
		if (font16x32[dv+12] & BIT(7-a)) ftmp[a] |= BIT(6);
		if (font16x32[dv+14] & BIT(7-a)) ftmp[a] |= BIT(7);
		ftmp[a+8] = 0;
		if (font16x32[dv+ 1] & BIT(7-a)) ftmp[a+8] |= BIT(0);
		if (font16x32[dv+ 3] & BIT(7-a)) ftmp[a+8] |= BIT(1);
		if (font16x32[dv+ 5] & BIT(7-a)) ftmp[a+8] |= BIT(2);
		if (font16x32[dv+ 7] & BIT(7-a)) ftmp[a+8] |= BIT(3);
		if (font16x32[dv+ 9] & BIT(7-a)) ftmp[a+8] |= BIT(4);
		if (font16x32[dv+11] & BIT(7-a)) ftmp[a+8] |= BIT(5);
		if (font16x32[dv+13] & BIT(7-a)) ftmp[a+8] |= BIT(6);
		if (font16x32[dv+15] & BIT(7-a)) ftmp[a+8] |= BIT(7);
		a++;
	}
	while(I2C_GetFlagStatus(I2C1, I2C_ISR_BUSY) != RESET){}; //   
	//  : , ,  1+16 , , 
	I2C_TransferHandling(I2C1, ADDR_LCD, 17, I2C_AutoEnd_Mode, I2C_Generate_Start_Write);
	WAIT_TXIS(); I2C_SendData(I2C1, DISP_LCD); //  
	for(a = 0; a < 16; a++) 
	{
		WAIT_TXIS(); I2C_SendData(I2C1, ftmp[a]);
	}
	WAIT_STOP();

	dv += 16; //  
	LCD_Cursor(X, 2); //   
	a = 0;
	while(a < 8)
	{
		ftmp[a] = 0;
		if (font16x32[dv+ 0] & BIT(7-a)) ftmp[a] |= BIT(0);
		if (font16x32[dv+ 2] & BIT(7-a)) ftmp[a] |= BIT(1);
		if (font16x32[dv+ 4] & BIT(7-a)) ftmp[a] |= BIT(2);
		if (font16x32[dv+ 6] & BIT(7-a)) ftmp[a] |= BIT(3);
		if (font16x32[dv+ 8] & BIT(7-a)) ftmp[a] |= BIT(4);
		if (font16x32[dv+10] & BIT(7-a)) ftmp[a] |= BIT(5);
		if (font16x32[dv+12] & BIT(7-a)) ftmp[a] |= BIT(6);
		if (font16x32[dv+14] & BIT(7-a)) ftmp[a] |= BIT(7);
		ftmp[a+8] = 0;
		if (font16x32[dv+ 1] & BIT(7-a)) ftmp[a+8] |= BIT(0);
		if (font16x32[dv+ 3] & BIT(7-a)) ftmp[a+8] |= BIT(1);
		if (font16x32[dv+ 5] & BIT(7-a)) ftmp[a+8] |= BIT(2);
		if (font16x32[dv+ 7] & BIT(7-a)) ftmp[a+8] |= BIT(3);
		if (font16x32[dv+ 9] & BIT(7-a)) ftmp[a+8] |= BIT(4);
		if (font16x32[dv+11] & BIT(7-a)) ftmp[a+8] |= BIT(5);
		if (font16x32[dv+13] & BIT(7-a)) ftmp[a+8] |= BIT(6);
		if (font16x32[dv+15] & BIT(7-a)) ftmp[a+8] |= BIT(7);
		a++;
	}
	while(I2C_GetFlagStatus(I2C1, I2C_ISR_BUSY) != RESET){}; //   
	//  : , ,  1+16 , , 
	I2C_TransferHandling(I2C1, ADDR_LCD, 17, I2C_AutoEnd_Mode, I2C_Generate_Start_Write);
	WAIT_TXIS(); I2C_SendData(I2C1, DISP_LCD); //  
	for(a = 0; a < 16; a++) 
	{
		WAIT_TXIS(); I2C_SendData(I2C1, ftmp[a]);
	}
	WAIT_STOP();
}


//----------------------------------------------------------------------------	
void LCD_Symbol(uint8_t codv) //       
{
	uint16_t w;

	w = codv * 6;

	while(I2C_GetFlagStatus(I2C1, I2C_ISR_BUSY) != RESET){}; //   
	//  : , ,  7 , , 
	I2C_TransferHandling(I2C1, ADDR_LCD, 7, I2C_AutoEnd_Mode, I2C_Generate_Start_Write);

	WAIT_TXIS(); I2C_SendData(I2C1, DISP_LCD); //  
	WAIT_TXIS(); I2C_SendData(I2C1, font[w++]);
	WAIT_TXIS(); I2C_SendData(I2C1, font[w++]);
	WAIT_TXIS(); I2C_SendData(I2C1, font[w++]);
	WAIT_TXIS(); I2C_SendData(I2C1, font[w++]);
	WAIT_TXIS(); I2C_SendData(I2C1, font[w++]);
	WAIT_TXIS(); I2C_SendData(I2C1, font[w++]);
	WAIT_STOP();
}

//----------------------------------------------------------------------------	
void LCD_String(char *str) //    	
{
	while(*str) LCD_Symbol(*str++);
}


//============================================================================================				

